"use strict";
/*
 * This code was provided by my teacher:
 *
 * Library code by Tom Hinkle
 * https://www.tomhinkle.net/proj/text-interface
 *
 * MIT License
 *
 * Copyright (c) [2024] [Thomas M. Hinkle]
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.TextInterface = void 0;
require("./textInterface.css");
const yesWords = ["yes", "yeah", "yep", "yup", "true", "t", "y", "aye", "yup"];
const noWords = ["no", "n", "false", "f", "nope", "nah"];
class TextInterface {
    constructor(element = document.body, title = "Text Interface") {
        this.listener = null;
        /*
         * the number of milliseconds taken to display
         * text passed to the output method
         */
        this.outputAnimationLength = 800;
        /*
         * The delay before outputting text
         */
        this.outputDelay = 300;
        this.outputQueue = [];
        this.div = document.createElement("div");
        this.div.classList.add("text-interface");
        element.appendChild(this.div);
        this.div.innerHTML = `
      <h2 class="ti-title">${title}</h2>
      <div class="output">
      </div>
      <div class="input-wrap">
          <div 
             class="input" 
             contenteditable 
             placeholder="Type here...">
          </div>
          <div class="placeholder">Type and hit return...</div>
      </div>
    `;
        this.inputWrap = this.div.querySelector(".input-wrap");
        this.inputEl = this.div.querySelector(".input");
        this.outputEl = this.div.querySelector(".output");
        this.placeholderEl = this.div.querySelector(".placeholder");
        this.setupInputListener();
        //this.inputEl.focus();
    }
    setTitle(text) {
        this.div.querySelector(".ti-title").textContent = text;
    }
    clear() {
        this.outputEl.innerHTML = "";
    }
    readChoice(choices, prompt = "Choose one of the following:", error = "You must choose one of the options!") {
        return __awaiter(this, void 0, void 0, function* () {
            this.output(prompt);
            for (let n = 0; n < choices.length; n++) {
                this.output(`${n + 1}. ${choices[n]}`);
            }
            let textInput = yield this.readText();
            // If they've typed the choice, just return the choice
            if (choices.indexOf(textInput) > -1) {
                return textInput;
            }
            // Otherwise, look for a number as an answer
            textInput = textInput.replace(/\D/g, "");
            if (textInput != "") {
                let number = Number(textInput);
                if (!isNaN(number) && number <= choices.length && number > 0) {
                    return choices[Math.floor(number) - 1];
                }
            }
            this.output(error);
            let correction = yield this.readChoice(choices, prompt, error);
            return correction;
        });
    }
    readYesOrNo(errorMessage = "Say yes or no!") {
        return __awaiter(this, void 0, void 0, function* () {
            let text = yield this.readText();
            text = text.toLowerCase();
            text = text.replace(/\s+/, "");
            if (yesWords.indexOf(text) > -1) {
                return true;
            }
            if (noWords.indexOf(text) > -1) {
                return false;
            }
            else {
                this.output(errorMessage);
                return yield this.readYesOrNo(errorMessage);
            }
        });
    }
    readNumber(errorMessage = "Please type a number") {
        return __awaiter(this, void 0, void 0, function* () {
            let text = yield this.readText();
            let number = Number(text);
            if (isNaN(number)) {
                this.output(errorMessage);
                return this.readNumber(errorMessage);
            }
            else {
                return number;
            }
        });
    }
    readText() {
        this.inputEl.focus();
        this.inputWrap.classList.add("active");
        this.inputWrap.scrollIntoView();
        return new Promise((resolve, reject) => {
            this.listener = resolve;
        });
    }
    showElement(element) {
        if (this.outputting) {
            this.outputQueue.push(["element", element]);
        }
        else {
            this.outputting = true;
            this.outputEl.appendChild(element);
            element.scrollIntoView({ behavior: "smooth" });
            setTimeout(() => {
                this.outputting = false;
                this.doNextOutput();
            }, this.outputDelay);
        }
    }
    showHTML(arbitraryHTML) {
        let div = document.createElement("div");
        div.innerHTML = arbitraryHTML;
        this.showElement(div);
    }
    showImage(src, alt = "An image") {
        let img = document.createElement("img");
        img.setAttribute("src", src);
        img.setAttribute("alt", alt);
        this.showElement(img);
    }
    output(text, echo = false) {
        text = "" + text;
        /* If we are already outputting, wait... */
        if (this.outputting) {
            this.outputQueue.push(["text", text, echo]);
        }
        else {
            let output = document.createElement("div");
            output.classList.add("output");
            if (echo) {
                output.classList.add("echo");
            }
            if (!this.outputAnimationLength || echo) {
                output.textContent = text;
            }
            else {
                this.outputting = true;
                let delay = this.outputAnimationLength / text.length;
                const animateOutput = () => {
                    output.textContent += text[0] || "";
                    text = text.substring(1);
                    if (text.length) {
                        setTimeout(animateOutput, delay);
                    }
                    else {
                        this.outputting = false;
                        this.doNextOutput();
                    }
                };
                setTimeout(animateOutput, this.outputDelay);
            }
            this.outputEl.appendChild(output);
            output.scrollIntoView({ behavior: "smooth" });
        }
    }
    doNextOutput() {
        if (this.outputQueue.length) {
            let next = this.outputQueue[0];
            this.outputQueue = this.outputQueue.slice(1);
            let nextMode = next[0];
            let nextArgs = next.slice(1);
            if (nextMode == "text") {
                this.output(...nextArgs);
            }
            else {
                // image
                this.showElement(...nextArgs);
            }
        }
    }
    setupInputListener() {
        this.inputEl.addEventListener("keypress", (event) => {
            let isEnter = event.code == "Enter";
            if (isEnter) {
                let input = this.inputEl.textContent.replace(/\n$/, "");
                this.output(input, true);
                if (this.listener) {
                    this.listener(input);
                    this.listener = null;
                }
                this.inputWrap.classList.remove("active");
                setTimeout(() => {
                    this.inputEl.textContent = "";
                }, 1); // after input fires
            }
        });
        this.placeholderEl.addEventListener("click", () => this.inputEl.focus());
    }
}
exports.TextInterface = TextInterface;
//# sourceMappingURL=textInterface.js.map